@@ -56,6 +56,8 @@ struct _SimpleFmtxWidgetPrivate
     gboolean is_boost_on;
     gboolean is_boost_installed;
     DBusConnection *bus;
+    GConfClient* gconf;
+    gboolean signals_connected;
 };
 static gboolean block_leave_notify = FALSE;
 
@@ -82,43 +84,42 @@ static void create_images(SimpleFmtxWidget *self);
 
 static void set_button_image(SimpleFmtxWidget *self, ButtonState state);
 static void open_settings_dialog(HDHomePluginItem *hdhomepluginitem, gpointer user_data);
-static void _launch_script(void);
+static void launch_script(void);
 static void set_gconf(SimpleFmtxWidget *self, gboolean boost);
 static gboolean get_gconf(SimpleFmtxWidget *self);
+static void visible_notify (GObject *object,
+                             GParamSpec *spec,
+                             gpointer data);
+static void update_ui(SimpleFmtxWidget* self);
 /*****************************************************************************************/
 
-static void _launch_script(void)
+static void launch_script(void)
 {
 
-    GFile *file = NULL;
-    file = g_file_new_for_path(FM_BOOST);
 
-    if(!g_file_query_exists(file, NULL))
-    {
-        hildon_banner_show_information(NULL, NULL, "Error while running script /sbin/fm-boost");
-    }
-    else
+    if(g_file_test(FM_BOOST, G_FILE_TEST_EXISTS))
     {
         system("/usr/bin/sudo /sbin/fm-boost");
         hildon_banner_show_information(NULL, NULL, "FM-Transmitter Power Level set to: 118");
     }
+    else
+    {
+        hildon_banner_show_information(NULL, NULL, "Error while running script /sbin/fm-boost");
+    }
 }
 static void set_gconf(SimpleFmtxWidget *self, gboolean boost)
 {
     SimpleFmtxWidgetPrivate *priv = SIMPLE_FMTX_WIDGET_GET_PRIVATE (self);
-    GConfClient* gconf = NULL;
-    gconf = gconf_client_get_default();
 
-    gconf_client_set_bool(gconf, GCONF_KEY_BOOST, boost, NULL);
+    gconf_client_set_bool(priv->gconf, GCONF_KEY_BOOST, boost, NULL);
     priv->is_boost_on = boost;
 }
 
 static gboolean get_gconf(SimpleFmtxWidget *self)
 {
     SimpleFmtxWidgetPrivate *priv = SIMPLE_FMTX_WIDGET_GET_PRIVATE (self);
-    GConfClient* gconf = NULL;
-    gconf = gconf_client_get_default();
-    priv->is_boost_on = gconf_client_get_bool(gconf, GCONF_KEY_BOOST,NULL);
+
+    priv->is_boost_on = gconf_client_get_bool(priv->gconf, GCONF_KEY_BOOST,NULL);
 
     return priv->is_boost_on;
 }   
@@ -272,10 +273,13 @@ static gboolean button_released (GtkWidget *widget,
     else if (priv->fmtx_state == FALSE)
     {
         set_button_image(user_data, ON);
-	    get_set((SimpleFmtxWidget*)user_data, SET, "state", TRUE);
+	    const gchar* error = get_set((SimpleFmtxWidget*)user_data, SET, "state", TRUE);
         if (priv->is_boost_on && priv->is_boost_installed)
-        {                    
-            _launch_script();
+        {   
+            if (g_strcmp0(error, "error") != 0)                 
+            {
+                launch_script();
+            }
         }
     }
         get_current_state(user_data);
@@ -399,11 +403,7 @@ static GtkWidget *build_ui (SimpleFmtxWidget *self)
 
 static gboolean is_fmtx_boost_installed(void)
 {
-
-    GFile *file = NULL;
-    file = g_file_new_for_path(FM_BOOST);
-
-    if(g_file_query_exists(file, NULL))
+    if(g_file_test(FM_BOOST, G_FILE_TEST_EXISTS))
     {
         return TRUE;
     }
@@ -411,7 +411,7 @@ static gboolean is_fmtx_boost_installed(void)
     {
         return FALSE;
     }
-}
+} 
 
 static gboolean is_boost_set(SimpleFmtxWidget *self)
 {
@@ -490,25 +490,24 @@ static void open_settings_dialog(HDHomePluginItem *hdhomepluginitem, gpointer us
     gtk_widget_destroy (GTK_WIDGET(settings_dialog));
     block_leave_notify = FALSE;
 }
+static void update_ui(SimpleFmtxWidget* self)
+{
+    set_freq_label(self);
+    get_current_state(self);
+}
 static DBusHandlerResult signal_filter(DBusConnection *connection, DBusMessage *message, void *user_data)
 {
-    set_freq_label((SimpleFmtxWidget*)user_data);
-    get_current_state((SimpleFmtxWidget*)user_data);
-
+    update_ui((SimpleFmtxWidget*)user_data);
     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
 static void set_dbus_listener(SimpleFmtxWidget *self)
 {
     SimpleFmtxWidgetPrivate *priv = SIMPLE_FMTX_WIDGET_GET_PRIVATE(self);
-    priv->bus = dbus_bus_get (DBUS_BUS_SYSTEM, NULL);
-    if (!priv->bus) 
-    {
-        return;
-    }
     dbus_connection_setup_with_g_main (priv->bus, NULL);
     dbus_bus_add_match (priv->bus, "type='signal',interface='com.nokia.FMTx.Device',path='/com/nokia/fmtx/default',member='Changed'", NULL);
     dbus_connection_add_filter (priv->bus, signal_filter, self, NULL);
+    priv->signals_connected = TRUE;
 }
 
 static void simple_fmtx_widget_plugin_init (SimpleFmtxWidget *desktop_plugin)
@@ -519,6 +518,8 @@ static void simple_fmtx_widget_plugin_init (SimpleFmtxWidget *desktop_plugin)
     GdkScreen *screen = NULL;
     GdkColormap *colormap = NULL;
 
+    priv->bus = (DBusConnection*) osso_get_sys_dbus_connection (priv->osso_context);
+    priv->gconf = gconf_client_get_default();
     /* Needed for transparency */
     screen = gtk_widget_get_screen(GTK_WIDGET(desktop_plugin));
     colormap = gdk_screen_get_rgba_colormap(screen);
@@ -543,16 +544,25 @@ static void simple_fmtx_widget_plugin_init (SimpleFmtxWidget *desktop_plugin)
     set_freq_label(desktop_plugin);
     get_current_state(desktop_plugin);
 
+    g_signal_connect (desktop_plugin, "notify::is-on-current-desktop",
+                      G_CALLBACK (visible_notify),
+                      desktop_plugin);
+
+
     gtk_container_add (GTK_CONTAINER (desktop_plugin), contents);
 }
 static void simple_fmtx_widget_plugin_finalize(GObject *object)
 {
     SimpleFmtxWidgetPrivate *priv = SIMPLE_FMTX_WIDGET_GET_PRIVATE (object);
+    if (priv->signals_connected  == TRUE)
+    {
+        dbus_connection_remove_filter(priv->bus, signal_filter,object);
+        priv->signals_connected = FALSE;
+    }
+
     osso_deinitialize(priv->osso_context);
     priv->osso_context = NULL;
-
-    dbus_connection_remove_filter(priv->bus, signal_filter,object);
-
+    g_object_unref(priv->gconf);
     G_OBJECT_CLASS (simple_fmtx_widget_plugin_parent_class)->finalize (object); 
 }
 
@@ -578,8 +588,6 @@ static gpointer get_set(SimpleFmtxWidget *self, PropertyCommand commad, const gc
     GValue value = { 0 };
     gchar* command = NULL;
 
-    SimpleFmtxWidgetPrivate *priv = SIMPLE_FMTX_WIDGET_GET_PRIVATE (self);
-
     connection = dbus_g_bus_get(DBUS_BUS_SYSTEM, &error);
     if (connection == NULL || error)
     {
@@ -693,7 +701,7 @@ static gpointer get_set(SimpleFmtxWidget *self, PropertyCommand commad, const gc
 		        hildon_banner_show_information(NULL, NULL, error->message);
 		        g_clear_error (&error);
 		        g_free(command);
-		        return NULL;
+		        return "error";
 		    }
 	    }
     }
@@ -701,3 +709,33 @@ static gpointer get_set(SimpleFmtxWidget *self, PropertyCommand commad, const gc
     return NULL;
 }
 
+/* Listen if desktop where widget is, is visible */
+static void visible_notify (GObject *object,
+                             GParamSpec *spec,
+                             gpointer data)
+{
+    gboolean is_visible;
+    SimpleFmtxWidgetPrivate *priv = SIMPLE_FMTX_WIDGET_GET_PRIVATE (data);
+
+    g_object_get (object, "is-on-current-desktop", &is_visible, NULL);
+
+    if(is_visible)
+    {
+       //visible
+        update_ui((SimpleFmtxWidget*)data);
+        if (priv->signals_connected == FALSE)
+        {
+            set_dbus_listener(data);
+        }
+    }
+    else
+    {
+       //not visible
+        if (priv->signals_connected  == TRUE)
+        {
+            dbus_connection_remove_filter(priv->bus, signal_filter,object);
+            priv->signals_connected = FALSE;
+        }
+    }
+
+}
